////////////////////////////////////////////////////////////////////////////////
//                                 
//                        (c)  , 1999-2000
//    http://mazurkin.virtualave.net, mazurkin@mailru.com, mazurkin@chat.ru
// _____________________________________________________________________________
//                               
////////////////////////////////////////////////////////////////////////////////

unit LssDialogSetup;

interface

uses
  Windows, Messages, LssWindowSaver, WinLite;

type
  TDialogSetup = class(TLiteDialogBox)
  private
    FParameters : TSaverParameters;
    FSaverWnd   : TWindowSaver;
  protected
    procedure   CreateDialogParams(var DialogParams: TDialogParams); override;
  protected
    procedure   OnSysCommand(var Msg: TWMSysCommand); message WM_SYSCOMMAND;
    procedure   OnInitDialog(var Msg: TWMInitDialog); message WM_INITDIALOG;
    procedure   OnDestroy(var Msg: TWMDestroy); message WM_DESTROY;
    procedure   OnCommand(var Msg: TWMCommand); message WM_COMMAND;
    procedure   OnScroll(var Msg: TWMHScroll); message WM_HSCROLL;
  public
    class procedure SetupRun(ParentWnd: THandle);
  end;

////////////////////////////////////////////////////////////////////////////////
//      
////////////////////////////////////////////////////////////////////////////////
function  LoadParameters: TSaverParameters;
procedure SaveParameters(const Parameters: TSaverParameters);

implementation

{$I res\resource.inc}

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
const
  SAVERKEY = 'SoftWare\NMS\WinLite Screen Saver';

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
class procedure TDialogSetup.SetupRun(ParentWnd: THandle);
var
  Dialog : TDialogSetup;
begin
  Dialog := TDialogSetup.Create(ParentWnd);
  Dialog.ShowModal;
  Dialog.Free;
end;

////////////////////////////////////////////////////////////////////////////////
// TDialogSetup
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TDialogSetup.CreateDialogParams(var DialogParams: TDialogParams);
begin
  inherited;
  DialogParams.Template := MAKEINTRESOURCE(DIALOGSETUP);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TDialogSetup.OnInitDialog(var Msg: TWMInitDialog);
begin
  inherited;
  //  
  FParameters := LoadParameters;
  //   
  SetScrollRange(GetDlgItem(WndHandle, IDC_SETUP_SPEED), SB_CTL, 1, 2000, False);
  //   
  SetScrollPos(GetDlgItem(WndHandle, IDC_SETUP_SPEED), SB_CTL, FParameters.PartsCount, True);
  //    
  FSaverWnd := TWindowSaver.CreateSaver(GetDlgItem(WndHandle, IDC_SETUP_BORDER), True);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TDialogSetup.OnDestroy(var Msg: TWMDestroy);
begin
  inherited;
  //    
  FSaverWnd.Free;
end;

////////////////////////////////////////////////////////////////////////////////
// TDialogSetup
// _____________________________________________________________________________
//   
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TDialogSetup.OnSysCommand(var Msg: TWMSysCommand);
begin
  //      ,   
  if Msg.CmdType = SC_CLOSE then
  begin
    EndDialog(WndHandle, IDOK);
    Msg.Result := 0;
    Exit;
  end;
  //     
  with TMessage(Msg) do
    DefWindowProc(WndHandle, Msg, WParam, LParam);
end;

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
procedure TDialogSetup.OnCommand(var Msg: TWMCommand);
begin
  //    "OK" -  
  if Msg.ItemID = IDC_SETUP_OK then
  begin
    //  
    SaveParameters(FParameters);
    //  
    EndDialog(WndHandle, IDOK);
    Msg.Result := 1;
  end;
  //    "Cancel" -  
  if Msg.ItemID = IDC_SETUP_CANCEL then
  begin
    EndDialog(WndHandle, IDCANCEL);
    Msg.Result := 1;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TDialogSetup.OnScroll(var Msg: TWMHScroll);
begin
  //     
  case Msg.ScrollCode of
    SB_BOTTOM        : FParameters.PartsCount := 0;
    SB_TOP           : FParameters.PartsCount := 100;
    SB_LINELEFT      : Dec(FParameters.PartsCount,  1);
    SB_LINERIGHT     : Inc(FParameters.PartsCount,  1);
    SB_PAGELEFT      : Dec(FParameters.PartsCount, 10);
    SB_PAGERIGHT     : Inc(FParameters.PartsCount, 10);
    SB_THUMBPOSITION : FParameters.PartsCount := Msg.Pos;
    SB_ENDSCROLL     : FSaverWnd.InitSaver(FParameters);
  end;
  //  
  if FParameters.PartsCount < 1    then FParameters.PartsCount := 1;
  if FParameters.PartsCount > 2000 then FParameters.PartsCount := 2000;
  //  
  SetScrollPos(Msg.ScrollBar, SB_CTL, FParameters.PartsCount, True);
end;

////////////////////////////////////////////////////////////////////////////////
// TDialogSetup
// _____________________________________________________________________________
//    
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
function LoadParameters: TSaverParameters;
var
  RK   : HKEY;
  Size : DWord;
begin
  //      
  if RegOpenKeyEx(HKEY_CURRENT_USER, SAVERKEY, 0, KEY_READ, RK) = ERROR_SUCCESS then
  begin
    //  
    Size := 4;
    RegQueryValueEx(RK, 'PartsCount', nil, nil, @Result, @Size);
    //  
    RegCloseKey(RK);
  end else
  begin
    //   
    Result.PartsCount := 1000;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure SaveParameters(const Parameters: TSaverParameters);
var
  RK : HKEY;
begin
  //  
  if RegCreateKeyEx(HKEY_CURRENT_USER, SAVERKEY, 0, nil, REG_OPTION_NON_VOLATILE,
       KEY_WRITE, nil, RK, nil) = ERROR_SUCCESS then
  begin
    //  
    RegSetValueEx(RK, 'PartsCount', 0, REG_DWORD, @Parameters.PartsCount, 4);
    //  
    RegCloseKey(RK);
  end;
end;

end.
