////////////////////////////////////////////////////////////////////////////////
//         WinLite,        Win32 API
//                       (c)  , 1999-2000
//    http://mazurkin.virtualave.net, mazurkin@mailru.com, mazurkin@chat.ru
// _____________________________________________________________________________
//                                 
////////////////////////////////////////////////////////////////////////////////

unit WinLite;

interface

uses Windows, Messages;

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
type
  TWindowParams = record
    Caption     : PChar;
    Style       : DWord;
    ExStyle     : DWord;
    X           : Integer;
    Y           : Integer;
    Width       : Integer;
    Height      : Integer;
    WndParent   : THandle;
    WndMenu     : THandle;
    Param       : Pointer;
    WindowClass : TWndClass;
  end;

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
type
  TDialogParams = record
    Template    : PChar;
    WndParent   : THandle;
  end;

////////////////////////////////////////////////////////////////////////////////
// TLiteFrame
// _____________________________________________________________________________
//     TLiteWindow, TLiteDialog, TLiteDialogBox
////////////////////////////////////////////////////////////////////////////////
type
  TLiteFrame = class(TObject)
  private
    FWndCallback: Pointer;
    FWndHandle  : THandle;
    FWndParent  : THandle;
    function    WindowCallback(hWnd: HWnd; Msg, WParam, LParam: Longint): Longint; stdcall;
  protected
    procedure   WindowProcedure(var Msg: TMessage); virtual;
  public
    property    WndHandle: THandle read FWndHandle;
    property    WndCallback: Pointer read FWndCallback;
  public
    constructor Create(AWndParent: THandle); virtual;
    destructor  Destroy; override;
  end;

////////////////////////////////////////////////////////////////////////////////
// TLiteWindow
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////
type
  TLiteWindow = class(TLiteFrame)
  private
    FWndParams  : TWindowParams;
    FWndSubclass: Pointer;
  protected
    procedure   CreateWindowParams(var WindowParams: TWindowParams); virtual;
  public
    procedure   DefaultHandler(var Msg); override;
    constructor Create(AWndParent: THandle); override;
    constructor CreateSubclassed(AWnd: THandle); virtual;
    destructor  Destroy; override;
  end;

////////////////////////////////////////////////////////////////////////////////
// TLiteDialog
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////
type
  TLiteDialog = class(TLiteFrame)
  private
    FDlgParams  : TDialogParams;
  protected
    procedure   CreateDialogParams(var DialogParams: TDialogParams); virtual;
  public
    procedure   DefaultHandler(var Msg); override;
    constructor Create(AWndParent: THandle); override;
    destructor  Destroy; override;
  end;

////////////////////////////////////////////////////////////////////////////////
// TLiteDialogBox
// _____________________________________________________________________________
//   
////////////////////////////////////////////////////////////////////////////////
type
  TLiteDialogBox = class(TLiteFrame)
  private
    FDlgParams  : TDialogParams;
  protected
    procedure   CreateDialogParams(var DialogParams: TDialogParams); virtual;
  public
    procedure   DefaultHandler(var Msg); override;
  public
    function    ShowModal: Integer; 
  end;

////////////////////////////////////////////////////////////////////////////////
// TLiteThread
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////
type
  TLiteThread = class(TObject)
  private
    FThrCallback: Pointer;
    FThrHandle  : THandle;
    FThrIdent   : DWord;
    function    ThreadCallback(Param: Pointer): DWord; stdcall;
  protected
    function    ThreadProcedure: DWord; virtual;
  public
    property    ThrHandle: THandle read FThrHandle;
    property    ThrIdent: DWord read FThrIdent;
  public
    constructor Create; virtual;
    destructor  Destroy; override;
  end;

implementation

////////////////////////////////////////////////////////////////////////////////
// TLiteFrame
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
constructor TLiteFrame.Create(AWndParent: THandle);
begin
  inherited Create;
  //    
  FWndParent := AWndParent;
  //      
  FWndCallback := VirtualAlloc(nil, 12, MEM_RESERVE OR MEM_COMMIT, PAGE_EXECUTE_READWRITE);
  //    
  asm
    mov  EAX, Self
    mov  ECX, [EAX].TLiteFrame.FWndCallback     
    mov  word  ptr [ECX+0], $6858               // pop  EAX
    mov  dword ptr [ECX+2], EAX                 // push _Self_
    mov  word  ptr [ECX+6], $E950               // push EAX
    mov  EAX, OFFSET(TLiteFrame.WindowCallback)
    sub  EAX, ECX
    sub  EAX, 12
    mov  dword ptr [ECX+8], EAX                 // jmp  TLiteFrame.WindowCallback
  end;
end;

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
destructor TLiteFrame.Destroy;
begin
  //     
  VirtualFree(FWndCallback, 0, MEM_RELEASE);
  //   
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteFrame
// _____________________________________________________________________________
//   
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////////
function TLiteFrame.WindowCallback(hWnd: HWnd; Msg, WParam, LParam: Integer): Longint;
var
  WindowMsg : TMessage;
begin
  //   ,      
  if FWndHandle = 0 then FWndHandle := hWnd;
  //  
  WindowMsg.Msg    := Msg;
  WindowMsg.WParam := WParam;
  WindowMsg.LParam := LParam;
  //  
  WindowProcedure(WindowMsg);
  //    
  Result := WindowMsg.Result;
end;

////////////////////////////////////////////////////////////////////////////////
//      
////////////////////////////////////////////////////////////////////////////////
procedure TLiteFrame.WindowProcedure(var Msg: TMessage);
begin
  //    
  Dispatch(Msg);
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteWindow
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
constructor TLiteWindow.Create(AWndParent: THandle);
begin
  inherited;
  //   
  CreateWindowParams(FWndParams);
  //   
  RegisterClass(FWndParams.WindowClass);
  //  
  with FWndParams do
    CreateWindowEx(ExStyle, WindowClass.lpszClassName, Caption,
      Style, X, Y, Width, Height,
      WndParent, WndMenu, hInstance, Param
    );
end;

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
constructor TLiteWindow.CreateSubclassed(AWnd: THandle);
begin
  inherited Create(GetParent(AWnd));
  //   
  FWndSubclass := Pointer(GetWindowLong(AWnd, GWL_WNDPROC));
  //   
  FWndHandle   := AWnd;
  //    
  SetWindowLong(FWndHandle, GWL_WNDPROC, DWord(WndCallback));
end;

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
destructor TLiteWindow.Destroy;
begin
  //   -   ?
  if FWndSubclass = nil then
  begin
    //   
    UnregisterClass(FWndParams.WindowClass.lpszClassName, hInstance);
    //  
    if IsWindow(FWndHandle) then DestroyWindow(FWndHandle);
  end
  else
    //    
    SetWindowLong(FWndHandle, GWL_WNDPROC, DWord(FWndSubclass));
  //   
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
procedure TLiteWindow.CreateWindowParams(var WindowParams: TWindowParams);
var
  WndClassName : string;
begin
  //   
  Str(DWord(Self), WndClassName);
  WndClassName := ClassName+':'+WndClassName;
  //     
  with FWndParams.WindowClass do
  begin
    style         := CS_DBLCLKS;
    lpfnWndProc   := WndCallback;
    cbClsExtra    := 0;
    cbWndExtra    := 0;
    lpszClassName := PChar(WndClassName);
    hInstance     := hInstance;
    hIcon         := LoadIcon(0, IDI_APPLICATION);
    hCursor       := LoadCursor(0, IDC_ARROW);
    hbrBackground := COLOR_BTNFACE + 1;
    lpszMenuName  := '';
  end;
  //    
  with FWndParams do
  begin
    WndParent := FWndParent;
    Caption := 'Lite Window';
    Style   := WS_OVERLAPPEDWINDOW or WS_VISIBLE;
    ExStyle := 0;
    X       := Integer(CW_USEDEFAULT);
    Y       := Integer(CW_USEDEFAULT);
    Width   := Integer(CW_USEDEFAULT);
    Height  := Integer(CW_USEDEFAULT);
    WndMenu := 0;
    Param   := nil;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteWindow
// _____________________________________________________________________________
//   
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
procedure TLiteWindow.DefaultHandler(var Msg);
begin
  //   -   ?
  if FWndSubclass = nil then
    //     
    with TMessage(Msg) do Result := DefWindowProc(FWndHandle, Msg, WParam, LParam)
  else
    //      
    with TMessage(Msg) do Result := CallWindowProc(FWndSubclass, FWndHandle, Msg, WParam, LParam);
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteDialog
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
constructor TLiteDialog.Create(AWndParent: THandle);
begin
  inherited;
  //   
  CreateDialogParams(FDlgParams);
  //  
  with FDlgParams do
    CreateDialogParam(hInstance, Template, WndParent, WndCallback, 0);
end;

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
destructor TLiteDialog.Destroy;
begin
  //  
  if IsWindow(FWndHandle) then DestroyWindow(FWndHandle);
  //   
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
procedure TLiteDialog.CreateDialogParams(var DialogParams: TDialogParams);
begin
  DialogParams.WndParent := FWndParent;
  DialogParams.Template  := '';
end;

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
procedure TLiteDialog.DefaultHandler(var Msg);
begin
  //    
  with TMessage(Msg) do
    if Msg = WM_INITDIALOG then Result := 1
                           else Result := 0;
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteDialogBox
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
procedure TLiteDialogBox.CreateDialogParams(var DialogParams: TDialogParams);
begin
  DialogParams.WndParent := FWndParent;
  DialogParams.Template  := '';
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
function TLiteDialogBox.ShowModal: Integer;
begin
  //   
  CreateDialogParams(FDlgParams);
  //  
  with FDlgParams do
    Result := DialogBoxParam(hInstance, Template, WndParent, WndCallback, 0);
end;

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
procedure TLiteDialogBox.DefaultHandler(var Msg);
begin
  //    
  with TMessage(Msg) do
    if Msg = WM_INITDIALOG then Result := 1
                           else Result := 0;
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteThread
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
constructor TLiteThread.Create;
begin
  inherited Create;
  //      
  FThrCallback := VirtualAlloc(nil, 12, MEM_RESERVE OR MEM_COMMIT, PAGE_EXECUTE_READWRITE);
  //    
  asm
    mov  EAX, Self
    mov  ECX, [EAX].TLiteThread.FThrCallback
    mov  word  ptr [ECX+0], $6858               // pop  EAX
    mov  dword ptr [ECX+2], EAX                 // push _Self_
    mov  word  ptr [ECX+6], $E950               // push EAX
    mov  EAX, OFFSET(TLiteThread.ThreadCallback)
    sub  EAX, ECX
    sub  EAX, 12
    mov  dword ptr [ECX+8], EAX                 // jmp  TLiteThread.ThreadCallback
  end;
  //  
  FThrHandle := CreateThread(nil, 0, FThrCallback, nil, CREATE_SUSPENDED, FThrIdent);
end;

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
destructor TLiteThread.Destroy;
begin
  //  
  CloseHandle(FThrHandle);
  //     
  VirtualFree(FThrCallback, 0, MEM_RELEASE);
  //   
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
// TLiteThread
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
function TLiteThread.ThreadCallback(Param: Pointer): DWord;
begin
  //    
  Result := ThreadProcedure;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
function TLiteThread.ThreadProcedure: DWord;
begin
  //   
  Result := 0;
end;

end.
